//////////////////////////////////////////////////////////////////////////////////
//																				//
//		RollAPI Header File														//
//		Copyright (C),1998 SoftLab-Nsk											//
//																				//
//		Author:		Egor Tarancev												//
//																				//
//////////////////////////////////////////////////////////////////////////////////

#ifndef RollAPI_HPP
#define RollAPI_HPP

#ifdef ROLLAPI_CPP
#define ROLLAPI_EXTERN
#else
#define ROLLAPI_EXTERN extern
#endif

//-- type declarations -------------------------------------------------------
typedef void* ImageHandle;  // Image
typedef void* ScreenHandle; // Screen
typedef void* RegionHandle; // Region

//-- const ---------------------------------------------------
//
#define ROLLAPI_VERSION ((long)(0x091))
//
// mode of allocation hardware resources for title playback
#define RollAPI_PlayDefaultB    (0) // try bank B, then try bank A
#define RollAPI_PlayInBankA     (1) // in bank A
#define RollAPI_PlayInBankB     (2) // in bank B
#define RollAPI_PlayDefaultA    (3) // try bank A, then try bank B
//
// mode of showing titles (mixer state changing)
#define RollAPI_ShowDontChange  (0) // don't change mixer state
#define RollAPI_ShowTitle       (1) // show title above another channel
#define RollAPI_ShowLive        (2) // hide title and show another channel
#define RollAPI_ShowDefault     (3) // use default behavior
//
// number of subdivision of one vertical pixel or two horizontal pixels
// for moving images in region
#define RollAPI_1xPrecision (0)
#define RollAPI_2xPrecision (1)
#define RollAPI_4xPrecision (2)
#define RollAPI_8xPrecision (3)
//#define RollAPI_16xPrecision (4)
//
//
// Errors
// all right
#define RollAPI_OK                  (0)
// errors
#define RollAPI_ErrNotRealized      (-1)
#define RollAPI_ErrNoMemory         (-2)
#define RollAPI_ErrNoTitleMemory    (-3)
#define RollAPI_ErrInvalidParameter (-4)
#define RollAPI_ErrInvalidFormat    (-5)
#define RollAPI_ErrInvalidHandle    (-6)
#define RollAPI_ErrAreaIsBusy       (-7)
#define RollAPI_ErrAreaIsEmpty      (-8)
#define RollAPI_ErrScreenExist      (-9)
#define RollAPI_ErrNoHardware       (-10)
#define RollAPI_ErrNoResources      (-11)
#define RollAPI_ErrNoFifoMemory     (-12)
#define RollAPI_ErrNotPrepared      (-13)
#define RollAPI_ErrNotPlayed        (-14)
#define RollAPI_ErrNotPaused        (-15)
#define RollAPI_ErrFatal            (-255)
// warnings
#define RollAPI_WarnEmptyResult     (1)


//-- procedures ---------------------------------------------------
//
#ifdef __cplusplus
extern "C" {
#endif
//
// -------------- Images ---------------- //
// make YUV image from whole RGB or RGBA bitmap
//  width must be aligned to 4 pixels
//  height must be aligned to 2 pixels
ROLLAPI_EXTERN ImageHandle (WINAPI * RollAPI_MakeImage)
               (BITMAPINFOHEADER *info, void *data);
//
// make compex YUV image from array of YUV 'images' ('cnt' - size of array)
//  dx,dy - size of new image (must be aligned to 4x2 pixels)
//  clip[i] - needed rectangle of 'images[i]' (if empty => whole image)
//  pos[i] - upper left corner of 'clip[i]' in new image
ROLLAPI_EXTERN ImageHandle (WINAPI * RollAPI_ComposeImage)
               (int dx, int dy, int cnt, POINT pos[],
                ImageHandle images[], RECT clip[]);
//
// delete YUV image (free memory)
ROLLAPI_EXTERN long (WINAPI * RollAPI_DeleteImage)
               (ImageHandle image);
//
// copy YUVA data to specified buffer - 4 bytes per pixel (Cb,Y,Cr,A).
// size of buffer (width and height) must be same as image sizes
// 'image' must be handle of simple image (not from RollAPI_ComposeImage)
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetImageData)
               (ImageHandle image, int dx, int dy, long *data);
//
// set new YUVA data from specified buffer - 4 bytes per pixel (Cb,Y,Cr,A).
// size of buffer (width and height) must be same as image sizes
// 'image' must be handle of simple image (not from RollAPI_ComposeImage)
ROLLAPI_EXTERN long (WINAPI * RollAPI_SetImageData)
               (ImageHandle image, int dx, int dy, long *data);
//
//
// ------------- Screen --------------- //
//
//
// allocate screen area on Forward device in bank 'play_bank':
// size of 'area' must be aligned to 4x2 pixels
// return screen coordinates into 'area' rectangle
ROLLAPI_EXTERN ScreenHandle (WINAPI * RollAPI_AllocateScreen)
               (RECT *area, long play_bank);
//
// delete screen and all it's regions (playback must be stopped)
ROLLAPI_EXTERN long (WINAPI * RollAPI_ReleaseScreen)
               (ScreenHandle screen);
//
// return hardware frame rate.
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetFrameRate)
               (double *lpFrameRate);
//
// return size & position of screen.
// if screen==NULL then return maximim avalible size of screen
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetScreenSize)
               (ScreenHandle screen, RECT *lpRect);
//
//
// show image on screen at specified position 'pos' (upper left corner)
ROLLAPI_EXTERN long (WINAPI * RollAPI_ShowImage)
               (ScreenHandle screen, ImageHandle image,
                POINT *pos, long show);
//
// copy image on memory (not to device) at position 'pos' (upper left corner)
ROLLAPI_EXTERN long (WINAPI * RollAPI_PutImage)
               (ScreenHandle screen, ImageHandle image, POINT *pos);
//
// clear (make transparent) specified rectangle on screen
ROLLAPI_EXTERN long (WINAPI * RollAPI_ClearArea)
               (ScreenHandle screen, RECT *area);
//
// copy screen data to device buffers and show it as 'show'
ROLLAPI_EXTERN long (WINAPI * RollAPI_ShowScreen)
               (ScreenHandle screen, long show);
//
// prepare all regions of this screen to playback titles
ROLLAPI_EXTERN long (WINAPI * RollAPI_PrepareScreenPlayback)
               (ScreenHandle screen);
// prepare all regions of this screen to playback titles
//  with specified maximum fifo filling number (at least 5 frames)
ROLLAPI_EXTERN long (WINAPI * RollAPI_PrepareScreenPlayback2)
               (ScreenHandle screen, long MaxFramesInFifo);
//
// function set for playback prepared regions of this screen:
//  show - describe showing mode (how to change mixer state)
ROLLAPI_EXTERN long (WINAPI * RollAPI_StartScreenPlayback)
               (ScreenHandle screen, long show);
ROLLAPI_EXTERN long (WINAPI * RollAPI_PauseScreenPlayback)
               (ScreenHandle screen, long show);
ROLLAPI_EXTERN long (WINAPI * RollAPI_UnpauseScreenPlayback)
               (ScreenHandle screen, long show);
ROLLAPI_EXTERN long (WINAPI * RollAPI_StopScreenPlayback)
               (ScreenHandle screen, long show);
ROLLAPI_EXTERN long (WINAPI * RollAPI_WaitStopScreenPlayback)
               (ScreenHandle screen, long show);
//
// return current frame (playback always starts from 0)
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetCurrentFrame)
               (ScreenHandle screen, int *lpFrameNumber);
//
// return working frame - last frame in FIFO (starting from 0)
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetWorkingFrame)
               (ScreenHandle screen, int *lpFrameNumber);
//
//
// ------------- Regiones --------------- //
//
// allocate region (rectangle) inside 'screen' with specified vertical or
//  horizontal moving. Size of 'area' must be aligned to 4x2 pixels.
// 2**'precision' specifies number of subdivision of moved titles.
// 'cnt' specifies number of phase of images, times[] - duration of each
//  phase in frames.
ROLLAPI_EXTERN RegionHandle (WINAPI * RollAPI_AllocateRegion)
               (ScreenHandle screen, RECT *area,
                double vSpeed, double hSpeed, long precision,
                int cnt, ImageHandle images[], long times[]);
//
// delete region from screen
ROLLAPI_EXTERN long (WINAPI * RollAPI_ReleaseRegion)
               (ScreenHandle screen, RegionHandle region);
//
// change playback speed scale (from 0.0 to 1.0)
ROLLAPI_EXTERN long (WINAPI * RollAPI_SetRegionSpeedScale)
               (RegionHandle region, double scale);
//
// return area, which is visible on hardware just now (and image index for animated region)
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetRegionPosition)
               (ScreenHandle screen, RegionHandle region,
			    RECT *lpRect, int *lpImageIndex);
//
// return area, which is prepared just now (and image index for animated region)
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetPreparedRegionPosition)
               (ScreenHandle screen, RegionHandle region,
			    RECT *lpRect, int *lpImageIndex);
//
typedef void (*RollAPICallbackProc)(void *param);
// set/clear region callback - it will be called after frame preparing
ROLLAPI_EXTERN long (WINAPI * RollAPI_SetRegionCallback)
               (RegionHandle region, RollAPICallbackProc proc, void *param);
//
//
// -------------- Errors ---------------- //
ROLLAPI_EXTERN long (WINAPI * RollAPI_GetLastError)
               (void);
//
#ifdef __cplusplus
}
#endif

#endif // _ROLL_API_
